@@ -19,8 +19,8 @@ function(sgp_object,
 
 	if (missing(state)) {
 		tmp.name <- gsub("_", " ", deparse(substitute(sgp_object)))
-		if (any(sapply(c(state.name, "Demonstration"), function(x) regexpr(x, tmp.name)))==1) {
-			state <- c(state.abb, "DEMO")[which(sapply(c(state.name, "Demonstration"), function(x) regexpr(x, tmp.name))==1)]
+		if (any(sapply(c(state.name, "Demonstration", "AOB"), function(x) regexpr(x, tmp.name))==1)) {
+			state <- c(state.abb, "DEMO", "AOB")[which(sapply(c(state.name, "Demonstration", "AOB"), function(x) regexpr(x, tmp.name))==1)]
 		}
 	}
 
@@ -46,9 +46,7 @@ function(sgp_object,
                         if (all(names(list_2[[j]]) %in% names(list_1[[j]]))) {
                                 for (k in names(list_2[[j]])) { # merging list_2 in with list_1, so use it here
                                         if (!identical(list_1[[j]][[k]], list_2[[j]][[k]])) { # keeps it from copying first set of results
-                                                if (dim(list_1[[j]][[k]])[2] != dim(list_2[[j]][[k]])[2]) {
-                                                        list_1[[j]][[k]] <- rbind.fill(list_1[[j]][[k]], list_2[[j]][[k]])
-                                                }       else list_1[[j]][[k]] <- rbind(list_1[[j]][[k]], list_2[[j]][[k]])
+                                               list_1[[j]][[k]] <- rbind.fill(list_1[[j]][[k]], list_2[[j]][[k]])
                                         }
                                 }
                         }
@@ -91,15 +89,16 @@ function(sgp_object,
 	                        timevar="GRADE",
 	                        direction="wide",
 	                        drop=names(sgp_object@Data)[!names(sgp_object@Data) %in% c('ID', 'SCALE_SCORE', 'GRADE')])
-	                        tmp.list[[k]] <- tmp.list[[k]][!apply(is.na(tmp.list[[k]]), 1, any)]
+	                        tmp.list[[k]] <- as.data.frame(tmp.list[[k]][!apply(is.na(tmp.list[[k]]), 1, any)])
 	        }
-	        tmp.dt <- do.call(rbind, tmp.list)
+	        tmp.df <- rbind.fill(tmp.list)
+
 
 	        ## Calculate Coefficient Matrices and return list containing coefficient matrix
 
 	        tmp_sgp_list <- list(Coefficient_Matrices =
 	                studentGrowthPercentiles(
-	                panel.data=data.frame(tmp.dt[,1,with=FALSE], matrix(grade.sequences, nrow=1), tmp.dt[,-1,with=FALSE]),
+	                panel.data=data.frame(tmp.df[,1], matrix(grade.sequences, nrow=1), tmp.df[,-1]),
 	                sgp.labels=list(my.year="BASELINE", my.subject=content_areas),
 	                use.my.knots.boundaries=state,
 	                calculate.sgps=FALSE,
@@ -118,7 +117,7 @@ function(sgp_object,
 	} ## END baselineSGP Function
 
         .get.config <- function(content_area, year, grades) {
-                tmp.data <- sgp_object@Data[J("VALID_CASE", content_area), c("YEAR", "GRADE"), with=FALSE]
+                tmp.data <- sgp_object@Data[SJ("VALID_CASE", content_area), c("YEAR", "GRADE"), with=FALSE]
                 tmp.unique.years <- sort(unique(tmp.data$YEAR))
                 .sgp.panel.years <- tmp.unique.years[1:which(tmp.unique.years == year)]
                 .sgp.content.areas <- rep(content_area, length(.sgp.panel.years))
@@ -153,8 +152,8 @@ function(sgp_object,
 	        if (missing(sgp.baseline.config)) {
  	               sgp.baseline.config <- tmp.sgp.baseline.config <- .content_areas <- .years <- .grades <- .sgp.grade.sequences <- list()
 	                       .content_areas <- unique(sgp_object@Data["VALID_CASE"][["CONTENT_AREA"]]) #tail(sgp.iter[["sgp.content.areas"]], 1)
-                               .years <- sort(unique(sgp_object@Data[J("VALID_CASE", .content_areas)][["YEAR"]]))
-                               .grades <- sort(unique(sgp_object@Data[J("VALID_CASE", .content_areas)][["GRADE"]]))
+                               .years <- sort(unique(sgp_object@Data[SJ("VALID_CASE", .content_areas)][["YEAR"]]))
+                               .grades <- sort(unique(sgp_object@Data[SJ("VALID_CASE", .content_areas)][["GRADE"]]))
                                .baseline.max.order <- length(.years)-2
                                tmp.sgp.grade.sequences <- lapply(.grades[-1], function(x) tail(.grades[.grades <= x], (.baseline.max.order+1)))
                                tmp.sgp.baseline.grade.sequences <- sapply(tmp.sgp.grade.sequences, function(x) x[(tail(x,1)-x) <= .baseline.max.order])
@@ -166,7 +165,7 @@ function(sgp_object,
                        sgp.baseline.grade.sequences <- unlist(sgp.baseline.grade.sequences, recursive=FALSE)
 
                        for (i in .content_areas) {
-                                tmp.sgp.baseline.config[[as.character(i)]] <- list(baseline.content.areas=i, baseline.panel.years=.years,
+                                tmp.sgp.baseline.config[[i]] <- list(baseline.content.areas=i, baseline.panel.years=.years,
                                         baseline.grade.sequences=sgp.baseline.grade.sequences)
                        }
 
@@ -216,7 +215,7 @@ function(sgp_object,
 	                }
 	                if (missing(years)) {
 	                        for (i in content_areas) {
-	                                tmp.years[[i]] <- sort(tail(unique(sgp_object@Data[J("VALID_CASE", i)][["YEAR"]]), -2), decreasing=TRUE)
+	                                tmp.years[[i]] <- sort(tail(unique(sgp_object@Data[SJ("VALID_CASE", i)][["YEAR"]]), -2), decreasing=TRUE)
 	                        }
 	                } else {
 	                        for (i in content_areas) {
@@ -226,7 +225,7 @@ function(sgp_object,
 	                if (missing(grades)) {
 	                        for (i in content_areas) {
 	                                for (j in tmp.years[[i]]) {
-	                                        tmp.grades[[paste(i,j,sep=".")]] <- sort(unique(sgp_object@Data[J("VALID_CASE", i, j)][["GRADE"]]))
+	                                        tmp.grades[[paste(i,j,sep=".")]] <- sort(unique(sgp_object@Data[SJ("VALID_CASE", i, j)][["GRADE"]]))
 	                                }
 	                        }
 	                } else {
@@ -247,7 +246,7 @@ function(sgp_object,
 
                 for (sgp.iter in sgp.config) {
                         tmp_sgp_object[["Panel_Data"]] <-
-                        as.data.frame(reshape(sgp_object@Data[J("VALID_CASE", sgp.iter[["sgp.content.areas"]], sgp.iter[["sgp.panel.years"]])],
+                        as.data.frame(reshape(sgp_object@Data[SJ("VALID_CASE", sgp.iter[["sgp.content.areas"]], sgp.iter[["sgp.panel.years"]])],
                                 idvar="ID",
                                 timevar="YEAR",
                                 drop=names(sgp_object@Data)[!names(sgp_object@Data) %in% c("ID", "GRADE", "SCALE_SCORE", "YEAR")],
