@@ -49,14 +49,21 @@
 require(gridBase)
 
 
+# Test for data to plot
+
+if (length(bubble_plot_data.X)==0) {
+	return("No data supplied for plotting. No plot produced")
+}
+
+
 # Test for installation of pdf2 package 
 
 if (bubble_plot_configs.BUBBLE_TIPS) {
-	if ("pdf2" %in% installed.packages() & as.numeric(version$minor) < 14) {
+	if (length(find.package("pdf2", quiet=TRUE)) > 0 & as.numeric(version$minor) < 14) {
 		require(pdf2)
 	} else {
 		bubble_plot_configs.BUBBLE_TIPS <- FALSE
-		message("Implentation of BUBBLE_TIPS requires the installation of the package pdf2 from R-Forge: install.packages('pdf2',repos='http://R-Forge.R-project.org')")
+		message("\tImplentation of BUBBLE_TIPS requires the installation of the package pdf2 from R-Forge: install.packages('pdf2',repos='http://R-Forge.R-project.org')")
 	}
 }
 
@@ -74,6 +81,11 @@ if (!is.null(bubble_plot_configs.BUBBLE_PLOT_PATH)) {
   
 # Calculate relevant quantities
 
+if (!is.null(bubble_plot_labels.SIZE)) {
+	numstud.range <- c(min(bubble_plot_labels.SIZE), max(bubble_plot_labels.SIZE))
+} else {
+	numstud.range <- c(25, 100)
+}
 num.sizes <- length(bubble_plot_labels.SIZE)
 if (!missing(bubble_plot_data.BUBBLE_TIPS_LINES)) num.bubble.lines <- length(bubble_plot_data.BUBBLE_TIPS_LINES)
 if (is.null(bubble_plot_data.LEVELS)) {
@@ -121,11 +133,12 @@ bubblecolor <- function(x){
 
 # Custom Bubble Size Function
 
-bubblesize <- function(schoolsize, numstud.range){
-                      slope <- (max.cex - min.cex)/(sqrt(numstud.range)[2] - sqrt(numstud.range)[1])
-                      temp <- slope*sqrt(schoolsize) - slope*sqrt(numstud.range)[2] + max.cex
-                      return(temp)
-}          
+bubblesize <- function(schoolsize, numstud.range) {
+		slope <- (max.cex - min.cex)/(sqrt(numstud.range)[2] - sqrt(numstud.range)[1])
+		temp <- slope*sqrt(schoolsize) - slope*sqrt(numstud.range)[2] + max.cex
+		temp[temp < min.cex] <- min.cex; temp[temp > max.cex] <- max.cex
+		return(temp)
+}
 
 
 # Custom Bubble Alpha Function
@@ -325,21 +338,22 @@ grid.text(x=0.95, y=0.92, paste("Higher",  bubble_plot_configs.BUBBLE_PLOT_BACKG
 
 # Add BUBBLE_PLOT_EXTRAS
 
+base.line <- "grid.lines(x=unit(50, 'native'), y=c(0.03,0.97), gp=gpar(col='grey40', lwd=1.25, lty=2, alpha=0.5))"
 if (!is.null(bubble_plot_configs.BUBBLE_PLOT_EXTRAS)) {
-   for (i in bubble_plot_configs.BUBBLE_PLOT_EXTRAS) {
+   for (i in c(base.line, bubble_plot_configs.BUBBLE_PLOT_EXTRAS)) {
       eval(parse(text=i))
    }
 } else {
-    grid.lines(x=unit(50, "native"), y=c(0.03,0.97), gp=gpar(col="grey40", lwd=1.5, lty=2, alpha=0.5))
+      eval(parse(text=base.line))
 }
 
 if (bubble_plot_configs.BUBBLE_TIPS) {
    if (!is.null(bubble_plot_data.SUBSET)) {
-      grid.circle(x=bubble_plot_data.X, y=bubble_plot_data.Y, r=unit(bubblesize(bubble_plot_data.SIZE, c(10,1000)), rep("inches", length(bubble_plot_data.SIZE))),
+      grid.circle(x=bubble_plot_data.X, y=bubble_plot_data.Y, r=unit(bubblesize(bubble_plot_data.SIZE, numstud.range), rep("inches", length(bubble_plot_data.SIZE))),
                gp=gpar(col=rgb(0.4,0.4,0.4), lwd=0.05*bubble_plot_configs.BUBBLE_MIN_MAX[2]/0.12, 
                fill=bubblecolor(unclass(tmp.LEVELS)), alpha=bubblealpha(length(bubble_plot_data.X), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Transparent)), default.units="native") 
       grid.circle(x=bubble_plot_data.X[bubble_plot_data.SUBSET], y=bubble_plot_data.Y[bubble_plot_data.SUBSET], 
-                  r=unit(bubble_plot_configs.BUBBLE_SUBSET_INCREASE+bubblesize(bubble_plot_data.SIZE[bubble_plot_data.SUBSET], c(10,1000)), 
+                  r=unit(bubble_plot_configs.BUBBLE_SUBSET_INCREASE+bubblesize(bubble_plot_data.SIZE[bubble_plot_data.SUBSET], numstud.range), 
                         rep("inches", length(bubble_plot_data.SIZE[bubble_plot_data.SUBSET]))), 
                   gp=gpar(lwd=0.750*bubble_plot_configs.BUBBLE_MIN_MAX[2]/0.12, fill=bubblecolor(unclass(tmp.LEVELS[bubble_plot_data.SUBSET])), 
                   alpha=bubblealpha(length(bubble_plot_data.X[bubble_plot_data.SUBSET]), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Opaque)), default.units="native") 
@@ -352,7 +366,7 @@ if (bubble_plot_configs.BUBBLE_TIPS) {
                             unit(bubble_plot_data.X[i], "native"), unit(bubble_plot_data.Y[i], "native"),
                             gp=gpar(lwd=0.5))
               grid.circle(x=bubble_plot_data.X[i], y=bubble_plot_data.Y[i], 
-                      r=unit(c(1.0, 0.4)*bubblesize(bubble_plot_data.SIZE[i], c(10,1000)), rep("inches", length(bubble_plot_data.SIZE[i]))), 
+                      r=unit(c(1.0, 0.4)*bubblesize(bubble_plot_data.SIZE[i], numstud.range), rep("inches", length(bubble_plot_data.SIZE[i]))), 
                       gp=gpar(lwd=c(0.5, 3.0), fill=bubblecolor(unclass(tmp.LEVELS))), default.units="native") 
               grid.rect(x=unit(indicate.coordinates$x, "npc"), y=unit(indicate.coordinates$y, "npc"), 
                              width=unit(2*text.buffer, "inches")+unit(1.0, "strwidth", bubble_plot_labels.BUBBLE_TITLES[i]), 
@@ -374,10 +388,10 @@ if (bubble_plot_configs.BUBBLE_TIPS) {
 
     if (!is.null(bubble_plot_data.BUBBLE_CENTER_LABEL)) {
             grid.text(x=bubble_plot_data.X, y=bubble_plot_data.Y, bubble_plot_data.BUBBLE_CENTER_LABEL, 
-                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, c(10,1000))/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
+                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, numstud.range)/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
                    alpha=bubblealpha(length(bubble_plot_data.X), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Transparent), font=2), default.units="native")
             grid.text(x=bubble_plot_data.X[bubble_plot_data.SUBSET], y=bubble_plot_data.Y[bubble_plot_data.SUBSET], bubble_plot_data.BUBBLE_CENTER_LABEL[bubble_plot_data.SUBSET], 
-                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, c(10,1000))/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
+                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, numstud.range)/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
                    alpha=bubblealpha(length(bubble_plot_data.X[bubble_plot_data.SUBSET]), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Opaque), font=2), default.units="native")
     }
 
@@ -392,7 +406,7 @@ if (bubble_plot_configs.BUBBLE_TIPS) {
                                                          bubble_plot_data.BUBBLE_TIPS_LINES[[j]][bubble_plot_data.SUBSET][i], sep=""))
           }
           text(0.5, 0.5, "X", col=rgb(1,0,0,0.01), popup="PLACEHOLDER", 
-               cex=10*bubblesize(bubble_plot_data.SIZE[bubble_plot_data.SUBSET][i], c(10,1000)),
+               cex=10*bubblesize(bubble_plot_data.SIZE[bubble_plot_data.SUBSET][i], numstud.range),
                annot.options=c(paste("/T (", bubble_plot_labels.BUBBLE_TITLES[bubble_plot_data.SUBSET][i], ")", sep=""), 
                         paste("/Contents (", paste(tmp.bubble.txt, collapse="\n"), ")", sep=""))) 
           popViewport()
@@ -402,7 +416,7 @@ if (bubble_plot_configs.BUBBLE_TIPS) {
    } ## End SUBSET if statement
 
    else {
-     grid.circle(x=bubble_plot_data.X, y=bubble_plot_data.Y, r=unit(bubblesize(bubble_plot_data.SIZE, c(10,1000)), rep("inches", length(bubble_plot_data.SIZE))), 
+     grid.circle(x=bubble_plot_data.X, y=bubble_plot_data.Y, r=unit(bubblesize(bubble_plot_data.SIZE, numstud.range), rep("inches", length(bubble_plot_data.SIZE))), 
                gp=gpar(col=rgb(0.2,0.2,0.2), lwd=0.05*bubble_plot_configs.BUBBLE_MIN_MAX[2]/0.12, 
                fill=bubblecolor(unclass(tmp.LEVELS)), alpha=bubblealpha(length(bubble_plot_data.X), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Opaque)), default.units="native") 
 
@@ -414,7 +428,7 @@ if (bubble_plot_configs.BUBBLE_TIPS) {
                             unit(bubble_plot_data.X[i], "native"), unit(bubble_plot_data.Y[i], "native"),
                             gp=gpar(lwd=0.5))
               grid.circle(x=bubble_plot_data.X[i], y=bubble_plot_data.Y[i], 
-                      r=unit(c(1.0, 0.4)*bubblesize(bubble_plot_data.SIZE[i], c(10,1000)), rep("inches", length(bubble_plot_data.SIZE[i]))), 
+                      r=unit(c(1.0, 0.4)*bubblesize(bubble_plot_data.SIZE[i], numstud.range), rep("inches", length(bubble_plot_data.SIZE[i]))), 
                       gp=gpar(lwd=c(0.5, 3.0), fill=bubblecolor(unclass(tmp.LEVELS))), default.units="native") 
               grid.rect(x=unit(indicate.coordinates$x, "npc"), y=unit(indicate.coordinates$y, "npc"), 
                              width=unit(2*text.buffer, "inches")+unit(1.0, "strwidth", bubble_plot_labels.BUBBLE_TITLES[i]), 
@@ -436,7 +450,7 @@ if (bubble_plot_configs.BUBBLE_TIPS) {
 
      if (!is.null(bubble_plot_data.BUBBLE_CENTER_LABEL)) {
             grid.text(x=bubble_plot_data.X, y=bubble_plot_data.Y, bubble_plot_data.BUBBLE_CENTER_LABEL, 
-                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, c(10,1000))/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
+                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, numstud.range)/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
                    alpha=bubblealpha(length(bubble_plot_data.X), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Opaque), font=2), default.units="native")
      }
 
@@ -451,7 +465,7 @@ if (bubble_plot_configs.BUBBLE_TIPS) {
                                                         bubble_plot_data.BUBBLE_TIPS_LINES[[j]][i], sep=""))
          }
          text(0.5, 0.5, "X", col=rgb(1,0,0,0.01), popup="PLACEHOLDER", 
-              cex=10*bubblesize(bubble_plot_data.SIZE[i], c(10,1000)),
+              cex=10*bubblesize(bubble_plot_data.SIZE[i], numstud.range),
               annot.options=c(paste("/T (", bubble_plot_labels.BUBBLE_TITLES[i], ")", sep=""), 
                         paste("/Contents (", paste(tmp.bubble.txt, collapse="\n"), ")", sep=""))) 
          popViewport()
@@ -464,11 +478,11 @@ if (bubble_plot_configs.BUBBLE_TIPS) {
 
 else {
    if (!is.null(bubble_plot_data.SUBSET)){
-      grid.circle(x=bubble_plot_data.X, y=bubble_plot_data.Y, r=unit(bubblesize(bubble_plot_data.SIZE, c(10,1000)), rep("inches", length(bubble_plot_data.SIZE))), 
+      grid.circle(x=bubble_plot_data.X, y=bubble_plot_data.Y, r=unit(bubblesize(bubble_plot_data.SIZE, numstud.range), rep("inches", length(bubble_plot_data.SIZE))), 
                gp=gpar(col=rgb(0.4,0.4,0.4), lwd=0.05*bubble_plot_configs.BUBBLE_MIN_MAX[2]/0.12, 
                fill=bubblecolor(unclass(tmp.LEVELS)), alpha=bubblealpha(length(bubble_plot_data.X), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Transparent)), default.units="native") 
       grid.circle(x=bubble_plot_data.X[bubble_plot_data.SUBSET], y=bubble_plot_data.Y[bubble_plot_data.SUBSET], 
-                  r=unit(bubble_plot_configs.BUBBLE_SUBSET_INCREASE+bubblesize(bubble_plot_data.SIZE[bubble_plot_data.SUBSET], c(10,1000)), 
+                  r=unit(bubble_plot_configs.BUBBLE_SUBSET_INCREASE+bubblesize(bubble_plot_data.SIZE[bubble_plot_data.SUBSET], numstud.range), 
                          rep("inches", length(bubble_plot_data.SIZE[bubble_plot_data.SUBSET]))), 
                   gp=gpar(lwd=0.75*bubble_plot_configs.BUBBLE_MIN_MAX[2]/0.12, fill=bubblecolor(unclass(tmp.LEVELS[bubble_plot_data.SUBSET])), 
                   alpha=bubblealpha(length(bubble_plot_data.X), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Opaque)), default.units="native") 
@@ -480,7 +494,7 @@ else {
                         unit(bubble_plot_data.X[bubble_plot_data.INDICATE], "native"), unit(bubble_plot_data.Y[bubble_plot_data.INDICATE], "native"),
                         gp=gpar(lwd=0.5))
           grid.circle(x=bubble_plot_data.X[bubble_plot_data.INDICATE], y=bubble_plot_data.Y[bubble_plot_data.INDICATE], 
-                  r=unit(c(1.0, 0.4)*bubblesize(bubble_plot_data.SIZE[bubble_plot_data.INDICATE], c(10,1000)), rep("inches", length(bubble_plot_data.SIZE[bubble_plot_data.INDICATE]))), 
+                  r=unit(c(1.0, 0.4)*bubblesize(bubble_plot_data.SIZE[bubble_plot_data.INDICATE], numstud.range), rep("inches", length(bubble_plot_data.SIZE[bubble_plot_data.INDICATE]))), 
                   gp=gpar(lwd=c(0.5, 3.0), fill=bubblecolor(unclass(tmp.LEVELS))), default.units="native") 
           grid.rect(x=unit(indicate.coordinates$x, "npc"), y=unit(indicate.coordinates$y, "npc"), 
                          width=unit(2*text.buffer, "inches")+unit(1.0, "strwidth", bubble_plot_labels.BUBBLE_TITLES[bubble_plot_data.INDICATE]), 
@@ -501,20 +515,20 @@ else {
 
    if (!is.null(bubble_plot_data.BUBBLE_CENTER_LABEL)) {
             grid.text(x=bubble_plot_data.X, y=bubble_plot_data.Y, bubble_plot_data.BUBBLE_CENTER_LABEL, 
-                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, c(10,1000))/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
+                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, numstud.range)/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
                    alpha=bubblealpha(length(bubble_plot_data.X), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Opaque), font=2), default.units="native")
    }
 
    if (!is.null(bubble_plot_data.BUBBLE_CENTER_LABEL)) {
             grid.text(x=bubble_plot_data.X, y=bubble_plot_data.Y, bubble_plot_data.BUBBLE_CENTER_LABEL, 
-                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, c(10,1000))/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
+                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, numstud.range)/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
                    alpha=bubblealpha(length(bubble_plot_data.X), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Transparent), font=2), default.units="native")
             grid.text(x=bubble_plot_data.X[bubble_plot_data.SUBSET], y=bubble_plot_data.Y[bubble_plot_data.SUBSET], bubble_plot_data.BUBBLE_CENTER_LABEL[bubble_plot_data.SUBSET], 
-                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, c(10,1000))/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
+                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, numstud.range)/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
                    alpha=bubblealpha(length(bubble_plot_data.X[bubble_plot_data.SUBSET]), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Opaque), font=2), default.units="native")
    }
 } else {  
-   grid.circle(x=bubble_plot_data.X, y=bubble_plot_data.Y, r=unit(bubblesize(bubble_plot_data.SIZE, c(10,1000)), rep("inches", length(bubble_plot_data.SIZE))), 
+   grid.circle(x=bubble_plot_data.X, y=bubble_plot_data.Y, r=unit(bubblesize(bubble_plot_data.SIZE, numstud.range), rep("inches", length(bubble_plot_data.SIZE))), 
                gp=gpar(col=rgb(0.2,0.2,0.2), lwd=0.05*bubble_plot_configs.BUBBLE_MIN_MAX[2]/0.12, fill=bubblecolor(unclass(tmp.LEVELS)), 
                alpha=bubblealpha(length(bubble_plot_data.X), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Opaque)), default.units="native") 
 
@@ -525,7 +539,7 @@ else {
                         unit(bubble_plot_data.X[bubble_plot_data.INDICATE], "native"), unit(bubble_plot_data.Y[bubble_plot_data.INDICATE], "native"),
                         gp=gpar(lwd=0.5))
           grid.circle(x=bubble_plot_data.X[bubble_plot_data.INDICATE], y=bubble_plot_data.Y[bubble_plot_data.INDICATE], 
-                  r=unit(c(1.0, 0.4)*bubblesize(bubble_plot_data.SIZE[bubble_plot_data.INDICATE], c(10,1000)), rep("inches", length(bubble_plot_data.SIZE[bubble_plot_data.INDICATE]))), 
+                  r=unit(c(1.0, 0.4)*bubblesize(bubble_plot_data.SIZE[bubble_plot_data.INDICATE], numstud.range), rep("inches", length(bubble_plot_data.SIZE[bubble_plot_data.INDICATE]))), 
                   gp=gpar(lwd=c(0.5, 3.0), fill=bubblecolor(unclass(tmp.LEVELS))), default.units="native") 
           grid.rect(x=unit(indicate.coordinates$x, "npc"), y=unit(indicate.coordinates$y, "npc"), 
                          width=unit(2*text.buffer, "inches")+unit(1.0, "strwidth", bubble_plot_labels.BUBBLE_TITLES[bubble_plot_data.INDICATE]), 
@@ -546,13 +560,13 @@ else {
 
    if (!is.null(bubble_plot_data.BUBBLE_CENTER_LABEL)) {
             grid.text(x=bubble_plot_data.X, y=bubble_plot_data.Y, bubble_plot_data.BUBBLE_CENTER_LABEL, 
-                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, c(10,1000))/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
+                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, numstud.range)/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
                    alpha=bubblealpha(length(bubble_plot_data.X), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Opaque), font=2), default.units="native")
    }
 
    if (!is.null(bubble_plot_data.BUBBLE_CENTER_LABEL)) {
             grid.text(x=bubble_plot_data.X, y=bubble_plot_data.Y, bubble_plot_data.BUBBLE_CENTER_LABEL, 
-                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, c(10,1000))/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
+                   gp=gpar(col=rgb(0.4,0.4,0.4), cex=bubblesize(bubble_plot_data.SIZE, numstud.range)/as.numeric(convertUnit(stringWidth(bubble_plot_data.BUBBLE_CENTER_LABEL), "inches")), 
                    alpha=bubblealpha(length(bubble_plot_data.X), bubble_plot_configs.BUBBLE_SUBSET_ALPHA$Opaque), font=2), default.units="native")
    }
 } ## End SUBSET else statement
@@ -577,7 +591,7 @@ grid.text(x=0.4, y=unit(text.start, "inches")+unit(text.buffer, "inches")+unit(0
 grid.text(x=0.4, y=unit(text.start+3*text.buffer, "inches")+unit(1.0, "strwidth", bubble_plot_labels.Y[1])+unit(0.5, "strwidth", bubble_plot_labels.Y[2]), 
           bubble_plot_labels.Y[2], 
           gp=gpar(col=format.colors.font[1]), rot=90, just="center")
-for (i in seq_along(bubble_plot_configs.BUBBLE_Y_TICKS)) {
+for (i in 2:(length(bubble_plot_configs.BUBBLE_Y_TICKS)-1)) {
      if (is.null(bubble_plot_configs.BUBBLE_Y_TICKS_SIZE)) {
           grid.text(x=0.925, y=bubble_plot_configs.BUBBLE_Y_TICKS[i], bubble_plot_configs.BUBBLE_Y_TICKS[i], 
                     gp=gpar(col=format.colors.font[1], cex=0.65), just=c("right", "center"), default.units="native")
@@ -638,7 +652,7 @@ if(!is.null(bubble_plot_data.SUBSET)) {
   bubble.legend.color <- rep(sort(my.colors)[1], length=num.sizes)
 }
 for (i in 1:num.sizes){
-grid.circle(x=0.25, y=y.coors[i], r=unit(bubblesize(bubble_plot_labels.SIZE[i], c(10,1000)), "inches"), 
+grid.circle(x=0.25, y=y.coors[i], r=unit(bubblesize(bubble_plot_labels.SIZE[i], numstud.range), "inches"), 
             gp=gpar(col="grey14", lwd=0.7, fill=bubble.legend.color[i], alpha=bubble.legend.alpha))
 grid.text(x=0.35, y=y.coors[i], paste(bubble_plot_labels.SIZE[i], "Students"), gp=gpar(col=format.colors.font[1], cex=0.9), just="left")
 }
