@@ -1,7 +1,8 @@
 `prepareSGP` <- 
 	function(data,
 		state=NULL,
-		var.names=NULL, 
+		var.names=NULL,
+		create.additional.variables=TRUE,
 		fix.duplicates="keep.all") {
 
 	## Print start time
@@ -9,14 +10,14 @@
 	started.at <- proc.time()
 	message(paste("\nStarted prepareSGP", date()))
 
-	VALID_CASE <- ID <- CONTENT_AREA <- YEAR <- ID <- GRADE <- SCALE_SCORE <- DUPLICATED_CASES <- NULL ## To prevent R CMD check warnings
+	VALID_CASE <- ID <- CONTENT_AREA <- YEAR <- ID <- GRADE <- SCALE_SCORE <- DUPLICATED_CASES <- SCALE_SCORE_PRIOR <- GRADE_PRIOR <- SCHOOL_NUMBER <- YEAR_INT <- NULL
 
 	## Get state (if possible)
 
 		if (is.null(state)) {
 			tmp.name <- gsub("_", " ", deparse(substitute(data)))
-			if (any(sapply(c(state.name, "Demonstration", "sgpData LONG"), function(x) regexpr(x, tmp.name)))==1) {
-				state <- c(state.abb, rep("DEMO", 2))[which(sapply(c(state.name, "Demonstration", "sgpData LONG"), function(x) regexpr(x, tmp.name))==1)]
+			if (any(sapply(c(state.name, "Demonstration", "sgpData LONG", "AOB"), function(x) regexpr(x, tmp.name))==1)) {
+				state <- c(state.abb, rep("DEMO", 2), "AOB")[which(sapply(c(state.name, "Demonstration", "sgpData LONG", "AOB"), function(x) regexpr(x, tmp.name))==1)]
 			} else {
 				state <- "TEMP"
 			}
@@ -25,12 +26,13 @@
 
 	### Utility functions
 
-	# achievement_level_recode (NOT YET IN USE)
+	# achievement_level_recode
 
 	achievement_level_recode <- function(sgp_object, state=NULL, year=NULL, content_area=NULL, grade=NULL) {
 		if (!"ACHIEVEMENT_LEVEL" %in% names(sgp_object@Data)) {
-			sgp_object@Data[["ACHIEVEMENT_LEVEL"]]<- factor(1, levels=seq_along(SGPstateData[[state]][["Achievement"]][["Levels"]][["Labels"]]),
-				labels=SGPstateData[[state]][["Achievement"]][["Levels"]][["Labels"]])
+			sgp_object@Data[["ACHIEVEMENT_LEVEL"]] <- 
+				factor(1, levels=seq_along(SGPstateData[[state]][["Achievement"]][["Levels"]][["Labels"]][!is.na(SGPstateData[[state]][["Achievement"]][["Levels"]][["Proficient"]])]),
+				labels=SGPstateData[[state]][["Achievement"]][["Levels"]][["Labels"]][!is.na(SGPstateData[[state]][["Achievement"]][["Levels"]][["Proficient"]])], ordered=TRUE)
 		}
 
 		if (is.null(year)) year <- sort(unique(sgp_object@Data$YEAR))
@@ -47,7 +49,7 @@
 					if (year==sort(c(year, tmp.cutscore.years))[1]) {
 						return(content_area)
 					} else {
-						return(paste(content_area, rev(sort(tmp.cutscore.years))[1], sep="."))
+						return(paste(content_area, sort(tmp.cutscore.years)[which(year==sort(c(year, tmp.cutscore.years)))-1], sep="."))
 					}
 				}
 			} else {
@@ -57,19 +59,39 @@
 
 		achievement_level_recode_INTERNAL <- function(state, content_area, year, grade, scale_score) {
 			factor(findInterval(scale_score, SGPstateData[[state]][["Achievement"]][["Cutscores"]][[get.cutscore.label(state, year, content_area)]][[paste("GRADE_", grade, sep="")]])+1,
-				levels=seq_along(SGPstateData[[state]][["Achievement"]][["Levels"]][["Labels"]]),
-				labels=SGPstateData[[state]][["Achievement"]][["Levels"]][["Labels"]])
+				levels=seq_along(SGPstateData[[state]][["Achievement"]][["Levels"]][["Labels"]][!is.na(SGPstateData[[state]][["Achievement"]][["Levels"]][["Proficient"]])]),
+				labels=SGPstateData[[state]][["Achievement"]][["Levels"]][["Labels"]][!is.na(SGPstateData[[state]][["Achievement"]][["Levels"]][["Proficient"]])], ordered=TRUE)
 		}
 
-		setkeyv(data, c("VALID_CASE", "CONTENT_AREA", "YEAR", "GRADE"))
+		setkeyv(sgp_object@Data, c("VALID_CASE", "CONTENT_AREA", "YEAR", "GRADE"))
 			sgp_object@Data[["ACHIEVEMENT_LEVEL"]][sgp_object@Data[CJ("VALID_CASE", content_area, year, grade), which=TRUE, nomatch=0]] <- 
-			sgp_object@Data[CJ("VALID_CASE", content_area, year, grade), nomatch=0][, achievement_level_recode_INTERNAL(state, as.character(CONTENT_AREA), as.character(YEAR), GRADE, SCALE_SCORE), 
+			sgp_object@Data[CJ("VALID_CASE", content_area, year, grade), nomatch=0][, achievement_level_recode_INTERNAL(state, CONTENT_AREA, YEAR, GRADE, SCALE_SCORE), 
 				by=list(CONTENT_AREA, YEAR, GRADE)][["V1"]]
-		setkeyv(data, c("VALID_CASE", "CONTENT_AREA", "YEAR", "ID"))
+		setkeyv(sgp_object@Data, c("VALID_CASE", "CONTENT_AREA", "YEAR", "ID"))
 
 		return(sgp_object)
 	}
 
+	# Function producing HIGH_NEED_STATUS variable (not yet in use)
+
+	my.quantile.function <- function(x, invalid_cases, quantiles=c(0.25, 0.75)) {
+		high.needs.status.labels <- c(paste("High Needs Status: Prior Achievement Below ", 100*quantiles[1], "th Percentile", sep=""),
+			NA, paste("High Needs Status: Prior Achievement Above ", 100*quantiles[2], "th Percentiles", sep=""))
+		if (invalid_cases) {
+			return(factor(rep(NA, length(x)), levels=1:2, labels=high.needs.status.labels[c(1,3)]))
+		}
+		if (all(is.na(x))) {
+			return(factor(rep(NA, length(x)), levels=1:2, labels=high.needs.status.labels[c(1,3)]))
+		} else {
+			my.quantiles <- quantile(x, probs=c(0, quantiles, 1), na.rm=TRUE)
+			if (any(diff(quantile(x, probs=c(0, quantiles, 1), na.rm=TRUE))==0)) {
+				return(factor(rep(NA, length(x)), levels=1:2, labels=high.needs.status.labels[c(1,3)]))
+			} else {
+				return(droplevels(cut(x, quantile(x, probs=c(0, quantiles, 1), na.rm=TRUE), include.lowest=TRUE, labels=high.needs.status.labels)))
+			}
+		}
+	} ### END my.quantile.function
+
 	## getNames
 
 	getNames <- function(data, var.names) {
@@ -152,16 +174,16 @@
 
 	create.knots.boundaries <- function(tmp.data) {
 		tmp.grade.list <- tmp.list <- list()
-		for (my.list.label in unique(tmp.data[J("VALID_CASE")][["CONTENT_AREA"]])) {
-			tmp.grade.list[[my.list.label]] <- unique(tmp.data[J("VALID_CASE", as.character(my.list.label))][["GRADE"]])
+		for (my.list.label in unique(tmp.data["VALID_CASE"][["CONTENT_AREA"]])) {
+			tmp.grade.list[[my.list.label]] <- unique(tmp.data[SJ("VALID_CASE", my.list.label)][["GRADE"]])
 			for (j in seq_along(tmp.grade.list[[my.list.label]])) {
 				tmp.list[[my.list.label]][[3*j-2]] <-
-					round(as.vector(quantile(subset(tmp.data, VALID_CASE=="VALID_CASE" & GRADE==tmp.grade.list[[my.list.label]][j], select="SCALE_SCORE"), 
+					round(as.vector(quantile(subset(tmp.data, VALID_CASE=="VALID_CASE" & CONTENT_AREA==my.list.label & GRADE==tmp.grade.list[[my.list.label]][j], select="SCALE_SCORE"), 
 						probs=c(0.2,0.4,0.6,0.8), na.rm=TRUE)), digits=3)
 				tmp.list[[my.list.label]][[3*j-1]] <-
-					round(as.vector(extendrange(subset(tmp.data, VALID_CASE=="VALID_CASE" & GRADE==tmp.grade.list[[my.list.label]][j], select="SCALE_SCORE"), f=0.1)), digits=3)
+					round(as.vector(extendrange(subset(tmp.data, VALID_CASE=="VALID_CASE" & CONTENT_AREA==my.list.label & GRADE==tmp.grade.list[[my.list.label]][j], select="SCALE_SCORE"), f=0.1)), digits=3)
 				tmp.list[[my.list.label]][[3*j]] <-
-					round(as.vector(extendrange(subset(tmp.data, VALID_CASE=="VALID_CASE" & GRADE==tmp.grade.list[[my.list.label]][j], select="SCALE_SCORE"), f=0.0)), digits=3)
+					round(as.vector(extendrange(subset(tmp.data, VALID_CASE=="VALID_CASE" & CONTENT_AREA==my.list.label & GRADE==tmp.grade.list[[my.list.label]][j], select="SCALE_SCORE"), f=0.0)), digits=3)
 			}
 			names(tmp.list[[my.list.label]]) <- paste(rep(c("knots_", "boundaries_", "loss.hoss_"), length(tmp.grade.list[[my.list.label]])), 
 				rep(tmp.grade.list[[my.list.label]], each=3), sep="")
@@ -202,18 +224,30 @@
 		}
 
 		if (!is.null(state) & is.null(var.names)) {
-			if (!identical(data@Names, SGPstateData[[state]][["Variable_Name_Lookup"]])) data@Names <- SGPstateData[[state]][["Variable_Name_Lookup"]]
+			if (!identical(state, "DEMO") & !identical(data@Names, SGPstateData[[state]][["Variable_Name_Lookup"]])) {
+				data@Names <- SGPstateData[[state]][["Variable_Name_Lookup"]]
+			} 
+			if (identical(state, "DEMO") & !identical(data@Names, SGPstateData[[state]][["Variable_Name_Lookup"]])) {
+				data@Names <- getNames(sgpData_LONG, var.names)
+			} 
 		}
+
 		if (!is.null(var.names)) {
 			data@Names <- getNames(data@Data, var.names)
 		}
 
+		## Check class values of fields
+
+		data <- checkSGP(data)
+
+
 		if (!identical(key(data@Data), c("VALID_CASE", "CONTENT_AREA", "YEAR", "ID"))) {
 			setkeyv(data@Data, c("VALID_CASE", "CONTENT_AREA", "YEAR", "ID"))
-			if (any(duplicated(data@Data[J("VALID_CASE")]))) {
+			if (any(duplicated(data@Data["VALID_CASE"]))) {
 				message("\tWARNING: @Data keyed by 'VALID_CASE', 'CONTENT_AREA', 'YEAR', 'ID' has duplicate cases. Subsequent merges will likely be corrupt.")
 				message("\tDuplicate cases are saved and available in current working environment as 'DUPLICATED_CASES'.")
-				assign("DUPLICATED_CASES", data[data[J("VALID_CASE")][duplicated(data[J("VALID_CASE")])][,list(VALID_CASE, CONTENT_AREA, YEAR, ID)]], envir=globalenv())
+				assign("DUPLICATED_CASES", data@Data["VALID_CASE"][duplicated(data@Data["VALID_CASE"])][,list(VALID_CASE, CONTENT_AREA, YEAR, ID)], envir=globalenv())
+				assign("DUPLICATED_CASES", data@Data["VALID_CASE"][duplicated(data@Data["VALID_CASE"])][,list(VALID_CASE, CONTENT_AREA, YEAR, ID)])
 				save(DUPLICATED_CASES, file="DUPLICATED_CASES.Rdata")
 			}
 		}
@@ -227,21 +261,21 @@
 		}
 
 		data@Version <- getVersion(data)
-
-		message(paste("Finished prepareSGP", date(), "in", timetaken(started.at), "\n"))
-		return(data)
+		sgp_object <- data
 	} else {
 		variable.names <- getNames(data, var.names)
+
 	
 		##  Create keyed data.table and check for duplicate cases
 
 		data <- as.data.table(data)
 		setnames(data, which(!is.na(variable.names$names.sgp)), variable.names$names.sgp[!is.na(variable.names$names.sgp)])
 		setkeyv(data, c("VALID_CASE", "CONTENT_AREA", "YEAR", "ID"))
-		if (any(duplicated(data[J("VALID_CASE")]))) {
+		if (any(duplicated(data["VALID_CASE"]))) {
 			message("\tWARNING: Data keyed by 'VALID_CASE', 'CONTENT_AREA', 'YEAR', 'ID' has duplicate cases. Subsequent merges will be corrupted.")
 			message("\tDuplicate cases are saved and available in current working environment as 'DUPLICATED_CASES'.")
-			assign("DUPLICATED_CASES", data[data[J("VALID_CASE")][duplicated(data[J("VALID_CASE")])][,list(VALID_CASE, CONTENT_AREA, YEAR, ID)]], envir=globalenv())
+			assign("DUPLICATED_CASES", data["VALID_CASE"][duplicated(data["VALID_CASE"])][,list(VALID_CASE, CONTENT_AREA, YEAR, ID)], envir=globalenv())
+			assign("DUPLICATED_CASES", data["VALID_CASE"][duplicated(data["VALID_CASE"])][,list(VALID_CASE, CONTENT_AREA, YEAR, ID)])
 			save(DUPLICATED_CASES, file="DUPLICATED_CASES.Rdata")
 		}
 
@@ -257,15 +291,61 @@
 		################################################################	
 		## INCLUDE CODE HERE TO HANDLE DUPLICATE CASES
 		################################################################	
-	
+
+		## Check class values of fields
+
+		data <- checkSGP(data)
+
 		##  Create the SGP object
 
 		sgp_object <- new("SGP", Data=data, Names=variable.names, Version=getVersion(data))
 
-	
-		##  Print finish time
-		message(paste("Finished prepareSGP", date(), "in", timetaken(started.at), "\n"))
-
-		return(sgp_object)
 	} ## END else
+
+	#################################################################
+	###
+	### Tidy up variables (could be validity checks)
+	###
+	#################################################################
+
+
+
+	#################################################################
+	###
+	### Add additional variables
+	###
+	#################################################################
+
+	## Create ACHIEVEMENT_LEVEL is it doesn't exist
+	
+	if (!"ACHIEVEMENT_LEVEL" %in% names(sgp_object@Data) & !is.null(SGPstateData[[state]][["Achievement"]][["Cutscores"]])) {
+		sgp_object <- achievement_level_recode(sgp_object, state=state)
+		message(paste("\tNOTE: Added variable ACHIEVEMENT_LEVEL to @Data using", state, "cutscores embedded in SGPstateData."))
+	}
+
+	if (create.additional.variables) {
+
+		### HIGH_NEED_STATUS
+
+		if (!"HIGH_NEED_STATUS" %in% names(sgp_object@Data) & "SCHOOL_NUMBER" %in% names(sgp_object@Data)) {
+			sgp_object@Data[["YEAR_INT"]] <- as.integer(factor(sgp_object@Data[["YEAR"]]))
+			setkeyv(sgp_object@Data, c("ID", "CONTENT_AREA", "YEAR_INT", "VALID_CASE")) ## CRITICAL that VALID_CASE is last in group
+			sgp_object@Data$SCALE_SCORE_PRIOR <- sgp_object@Data[SJ(ID, CONTENT_AREA, YEAR_INT-1L), mult="last"][,SCALE_SCORE]
+			sgp_object@Data$GRADE_PRIOR <- sgp_object@Data[SJ(ID, CONTENT_AREA, YEAR_INT-1L), mult="last"][,GRADE]
+
+			setkeyv(sgp_object@Data, c("VALID_CASE", "CONTENT_AREA", "YEAR_INT", "SCHOOL_NUMBER", "GRADE_PRIOR", "ID"))
+			sgp_object@Data[["HIGH_NEED_STATUS"]] <- sgp_object@Data[,my.quantile.function(SCALE_SCORE_PRIOR, !VALID_CASE[1]=="VALID_CASE"), 
+				by=list(VALID_CASE, CONTENT_AREA, YEAR_INT, SCHOOL_NUMBER, GRADE_PRIOR)]$V1
+			sgp_object@Data[["SCALE_SCORE_PRIOR"]] <- sgp_object@Data[["GRADE_PRIOR"]] <- sgp_object@Data[["YEAR_INT"]] <- NULL
+			message("\tNOTE: Added variable HIGH_NEED_STATUS to @Data.")
+			sgp_object@Names <- rbind(sgp_object@Names, c("HIGH_NEED_STATUS", "HIGH_NEED_STATUS", "demographic", "High need status flag", TRUE))
+		}
+
+	}
+
+
+	##  Print finish time
+	message(paste("Finished prepareSGP", date(), "in", timetaken(started.at), "\n"))
+
+	return(sgp_object)
 } ## END prepareSGP function
